Index: include/speex/speex_preprocess.h
===================================================================
--- include/speex/speex_preprocess.h	(revision 10333)
+++ include/speex/speex_preprocess.h	(working copy)
@@ -59,7 +59,9 @@
    float  reverb_level;
    float  speech_prob_start;
    float  speech_prob_continue;
-   
+   int adaptation_time;
+   int consec_noise_duration; /**< Number of consecutive noise frames required before considered speech*/
+
    float *frame;             /**< Processing frame (2*ps_size) */
    float *ps;                /**< Current power spectrum */
    float *gain2;             /**< Adjusted gains */
@@ -121,8 +123,9 @@
 /** Used like the ioctl function to control the preprocessor parameters */
 int speex_preprocess_ctl(SpeexPreprocessState *st, int request, void *ptr);
 
+/** Resets a denoising state */
+void speex_preprocess_state_reset(SpeexPreprocessState *st);
 
-
 /** Set preprocessor denoiser state */
 #define SPEEX_PREPROCESS_SET_DENOISE 0
 /** Get preprocessor denoiser state */
@@ -164,6 +167,12 @@
 #define SPEEX_PREPROCESS_SET_PROB_CONTINUE 16
 #define SPEEX_PREPROCESS_GET_PROB_CONTINUE 17
 
+#define SPEEX_PREPROCESS_SET_ADAPTATION_TIME 18
+#define SPEEX_PREPROCESS_GET_ADAPTATION_TIME 19
+
+#define SPEEX_PREPROCESS_SET_MIN_NOISE_DURATION 20
+#define SPEEX_PREPROCESS_GET_MIN_NOISE_DURATION 21
+
 #ifdef __cplusplus
 }
 #endif
Index: libspeex/mdf.c
===================================================================
--- libspeex/mdf.c	(revision 10333)
+++ libspeex/mdf.c	(working copy)
@@ -205,7 +205,8 @@
    float Syy=0,See=0;
    float leak_estimate;
    float ss;
-   
+   float adapt_rate;
+
    N = st->window_size;
    M = st->M;
    scale = 1.0f/N;
@@ -314,7 +315,6 @@
       /*printf ("%f\n", leak_estimate);*/
    }
    
-   float adapt_rate;
    if (!st->adapted)
    {
       float Sxx;
Index: libspeex/preprocess.c
===================================================================
--- libspeex/preprocess.c	(revision 10333)
+++ libspeex/preprocess.c	(working copy)
@@ -205,6 +205,19 @@
          st->window[i+N3]=1;
       }
    }
+   speex_preprocess_state_reset(st);
+
+   st->fft_lookup = (struct drft_lookup*)speex_alloc(sizeof(struct drft_lookup));
+   spx_drft_init(st->fft_lookup,2*N);
+   return st;
+}
+
+void speex_preprocess_state_reset(SpeexPreprocessState* st)
+{
+   int N, N3, i;
+   N = st->ps_size;
+   N3 = 2*N - st->frame_size;
+
    for (i=0;i<N;i++)
    {
       st->noise[i]=1e4;
@@ -223,7 +236,7 @@
 
    for (i=0;i<N;i++)
    {
-      float ff=((float)i)*.5*sampling_rate/((float)N);
+      float ff=((float)i)*.5*st->sampling_rate/((float)N);
       st->loudness_weight[i] = .35f-.35f*ff/16000.f+.73f*exp(-.5f*(ff-3800)*(ff-3800)/9e5f);
       if (st->loudness_weight[i]<.01f)
          st->loudness_weight[i]=.01f;
@@ -235,14 +248,9 @@
    st->loudness = pow(6000,LOUDNESS_EXP);
    st->loudness2 = 6000;
    st->nb_loudness_adapt = 0;
-
-   st->fft_lookup = (struct drft_lookup*)speex_alloc(sizeof(struct drft_lookup));
-   spx_drft_init(st->fft_lookup,2*N);
-
    st->nb_adapt=0;
    st->consec_noise=0;
-   st->nb_preprocess=0;
-   return st;
+   st->nb_preprocess=0;
 }
 
 void speex_preprocess_state_destroy(SpeexPreprocessState *st)
@@ -605,7 +613,7 @@
          st->Smin[i] = st->Stmp[i] = st->S[i]+100.f;
    }
 
-   if (st->nb_preprocess%200==0)
+   if (st->nb_preprocess%st->adaptation_time==0)
    {
       for (i=1;i<N-1;i++)
       {
@@ -738,7 +746,7 @@
       is_speech = speex_compute_vad(st, ps, mean_prior, mean_post);
 
 
-   if (st->consec_noise>=3)
+   if (st->consec_noise>=st->consec_noise_duration)
    {
       update_noise(st, st->old_ps, echo);
    } else {
@@ -1017,6 +1025,21 @@
    case SPEEX_PREPROCESS_GET_PROB_CONTINUE:
       (*(int*)ptr) = st->speech_prob_continue * 100;
       break;
+	case SPEEX_PREPROCESS_SET_MIN_NOISE_DURATION:
+		st->consec_noise_duration = (*(int*)ptr);
+		break;
+
+	case SPEEX_PREPROCESS_GET_MIN_NOISE_DURATION:
+		(*(int*)ptr) = st->consec_noise_duration;
+		break;
+
+	case SPEEX_PREPROCESS_SET_ADAPTATION_TIME:
+		st->adaptation_time = (*(int*)ptr);
+		break;
+
+	case SPEEX_PREPROCESS_GET_ADAPTATION_TIME:
+		(*(int*)ptr) = st->adaptation_time;
+		break;
 
       default:
       speex_warning_int("Unknown speex_preprocess_ctl request: ", request);